import sys
import re

reBF = re.compile(r'\(function \(([^)]*)\)*\)\s+{') # => /*<BF>*/(function (\1) { /*</BF>*/
reOB = re.compile(r'(.*){$')                        # => \1/*<OB>*/{/*</OB>*/
reCB = re.compile(r'(.*)}$')                        # => \1/*<CB>*/}/*</CB>*/
reEF = re.compile(r'}\)\((.*?)\);?$')               # => /*<EF>*/})(\1);/*</EF>*/
reL  = re.compile(r'(.*)')                          # => /*<L>*/\1/*</L>*/
reIC = re.compile(r'//(.*)')

def jsencode(program, level, inputfh):

    for line in inputfh:

        subprogramtext = "" # set to null string to free memory

        line = line.strip()

        if not line:
            continue

        if line[-1] == '/': # append a space to prevent inadvertent //
            line = line + ' '


        match = reBF.match(line)
        if match:
            if match.group(1):
                args = match.group(1)
            else:
                args = ''
            subprogram = ["/*<BF>*/(function (%s) {/*</BF>*/" % args]
            match = reIC.search(line)
            if match:
                subprogram.append("/* %s */" % match.group(1))

            jsencode(subprogram, level + 1, inputfh)
            subprogramtext = ' '.join(subprogram)
            if level == 0:
                print subprogramtext
            else:
                program.append(subprogramtext)
            continue

        match = reEF.match(line)
        if match:
            program.append("/*<EF>*/})(%s);/*</EF>*/" % match.group(1))
            return

        match = reOB.match(line)
        if match:
            subprogramtext = "/*<L>*/ %s /*<OB>*/{/*</OB>*/ /*</L>*/" % match.group(1)
            if level == 0:
                print subprogramtext
            else:
                program.append(subprogramtext)
            continue

        match = reCB.match(line)
        if match:
            subprogramtext = "/*<L>*/ %s /*<CB>*/}/*</CB>*/ /*</L>*/" % match.group(1)
            if level == 0:
                print subprogramtext
            else:
                program.append(subprogramtext)
            continue

        # handle inline comments specially
        line = reIC.sub(r'/*\1*/', line)

        subprogramtext = "/*<L>*/%s/*</L>*/" % line
        if level == 0:
            print subprogramtext
        else:
            program.append(subprogramtext)

if __name__ == "__main__":

    program   = []
    inputfile = sys.argv[1]

    inputfh = open(inputfile, 'r')

    jsencode(program, 0,  inputfh)

    inputfh.close()
