import sys
import re

filename = sys.argv[1]

stack = []
reArgs = re.compile(r'// args: (.*)')
reOpenBlock = re.compile(r'/\* open block ([0-9]+) \*/')
reCloseBlock = re.compile(r'/\* close block ([0-9]+) \*/')

line_number = 0
fh = open(filename, 'r')

# keep the function declaration line and don't immediately output it
# so we can output the block id with the function declaration to aid
# in debugging.

function_decl_line = ""

for line in fh:

    line = line.rstrip()

    line_number += 1

    match = reArgs.search(line)
    if match:
        stack.append({ 'args' : match.group(1)})
        function_decl_line = line
        # Delay outputing the function declaration line
        # until we have a block id.
        # The next line *must* be the open block comment.
    else:
        match = reOpenBlock.search(line)
        if match:

            id = match.group(1)
            if len(stack) > 0 and 'args' in stack[-1] and 'id' not in stack[-1]:
                stack[-1]['id'] = id
                print "%s block %s" % (function_decl_line, id)
                function_decl_line = ""
            else:
                if function_decl_line:
                    raise Exception("error: function declaration line %s missing args block line: %s at line %d" % (function_decl_line, line, line_number))
                stack.append({'id' : id})
        else:
            if function_decl_line:
                raise Exception("error: function declaration line %s not followed by open block line: %s at line %d" % (function_decl_line, line, line_number))

            match = reCloseBlock.search(line)
            if match:
                id = match.group(1)
                opened_block   = stack.pop()
                if opened_block['id'] != id:
                    raise Exception("error: opened block %s does not match closed block %s at line %d" % (opened_block['id'], id, line_number))
            else:
                print line

fh.close()

if len(stack) > 0:
    print "%s// CRASHED" % (len(stack)*"  ")

while len(stack) > 0:
    block = stack.pop()
    if 'args' in block:
        args = block['args']
        if args == 'none':
            args = ''
        print "%s})(%s);" % (len(stack)*"  ", args)
    else:
        print "%s}" % (len(stack)*"  ")


