#!/bin/bash

lithium=$1
firefox=$2

echo $lithium
echo $firefox

if [[ -z "$lithium" || -z "$firefox" ]]; then
    echo "Usage: run-lithium.sh pathtolithium pathtofirefox"
    exit 2
fi

cross_fuzz_dir=`pwd`

echo $cross_fuzz_dir

let level=0

trap 'cd $cross_fuzz_dir; exit 2' TERM INT

while true; do 

    pushd $lithium

    if [[ ! -e lithium.py ]]; then
	echo "Lithium not found"
	exit 2
    fi

    if [[ -e binarysearch.py ]]; then # new version of lithium
        if ! ./lithium.py ./crashes.py 300 $firefox $cross_fuzz_dir/test.html; then
            popd
            exit
        fi
    else
        if ! ./lithium.py ./crashes.py $cross_fuzz_dir/test.html $firefox; then
            popd
            exit
        fi
    fi

    popd

    let level=level+1

    echo "Extracting test script from test.html..."

    python extract-test-js.py test.html > temp-$level.js

    if [[ ! -s temp-$level.js ]]; then
        echo "Extracted script is empty"
        exit
    fi

    echo "Expanding test script to level $level..."

    if ! python expand-test-js.py temp-$level.js $level > test.js; then
        break
    fi

    cat testhead.html test.js testtail.html > test.html

done

# use extended regexp in sed
if [[ `uname` == "Darwin" ]]; then
    E='-E'
else
    E='-r'
fi

sed $E 's|/\*</?[^>]*>\*/||g' test.js > temp.js
cat testhead.html temp.js testtail.html > test.html
